#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Exceptions for specific hiera issues."""

from __future__ import print_function, unicode_literals


class HieraError(Exception):
    """Generic Hiera error."""

    def __init__(self, message, returncode=None, output=None):
        """Override instance init so that return code and console output can be
        added to error.
        """
        super(HieraError, self).__init__(message)
        self.returncode = returncode
        self.output = output


class HieraNotFoundError(Exception):
    """Hiera error indicating the hiera CLI could not be found."""
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Verify hiera.HieraClient class."""

from __future__ import print_function, unicode_literals

import mock
import subprocess
import unittest
import uuid

import hiera
import hiera.exc


class HieraClientTests(unittest.TestCase):
    __doc__

    def create_client(self, *args, **kwargs):
        """Helper to create a new hiera client.

        By default, patches _validate function so a non-existent config file
        may be used.

        :param patch_validate: bool optional switch to enable _validate patch
        :rtype: :class:`hiera.HieraClient`
        """
        patch_validate = kwargs.pop('patch_validate', True)
        if patch_validate:
            with mock.patch.object(hiera.HieraClient, '_validate'):
                return hiera.HieraClient(*args, **kwargs)
        else:
            return hiera.HieraClient(*args, **kwargs)

    def test_init(self):
        """Verify init with default params is successful."""
        h = self.create_client('my-config.yml')
        self.assertEqual('my-config.yml', h.config_filename)
        self.assertEqual('hiera', h.hiera_binary)
        self.assertEqual({}, h.environment)

    def test_init__environment(self):
        """Verify init stores all extra keyword arguments as environment
        variables.
        """
        expected_env = {'environment': 'unittest',
                        'host':        'ima-superstar',
                        'random_key':  'these-flashing-lights-are-bright!',
                        }
        h = self.create_client('my-config.yml',
                               environment='unittest',
                               host='ima-superstar',
                               random_key='these-flashing-lights-are-bright!')
        self.assertEqual(expected_env, h.environment)

    def test_init__nonexistent_config(self):
        """Verify HieraError is raised when config file does not exist."""
        with self.assertRaises(hiera.exc.HieraError):
            self.create_client(
                '/path/to/a/fake/config/{0}'.format(uuid.uuid4()),
                patch_validate=False)

    def test_repr(self):
        """Simple smoke test that verifies __repr__ is not busted."""
        h = self.create_client('my-config.yml')
        self.assertIsNotNone(str(h))

    @mock.patch.object(hiera.HieraClient, '_command')
    @mock.patch('subprocess.check_output')
    def test_hiera(self, mock_sub, mock_command):
        """Verify hiera returns output of subprocess command when
        successful.
        """
        h = self.create_client('my-config.yml')
        mock_sub.return_value = '"some-value"'

        actual_value = h._hiera('some-key')

        self.assertEqual('some-value', actual_value)
        mock_sub.assert_called_once_with(mock_command.return_value,
                                         stderr=subprocess.STDOUT)

    @mock.patch('subprocess.check_output')
    def test_hiera__whitespace(self, mock_sub):
        """Verify hiera strips whitespace from suprocess output when
        successful.
        """
        h = self.create_client('my-config.yml')
        mock_sub.return_value = '  \t\n\r\n"some-value"   '

        actual_value = h._hiera('some-key')

        self.assertEqual('some-value', actual_value)

    @mock.patch('subprocess.check_output')
    def test_hiera__missing_hiera(self, mock_sub):
        """Verify HieraNotFoundError is raised when hiera binary is not
        found.
        """
        h = self.create_client('my-config.yml')
        mock_sub.side_effect = OSError('kaboom!')

        with self.assertRaises(hiera.exc.HieraNotFoundError):
            h._hiera('some-key')

    @mock.patch('subprocess.check_output')
    def test_hiera__failed(self, mock_sub):
        """Verify HieraError raised when hiera subprocess command fails."""
        h = self.create_client('my-config.yml')
        mock_sub.side_effect = subprocess.CalledProcessError(
            1, ['command here'], output=None)

        with self.assertRaises(hiera.exc.HieraError):
            h._hiera('some-key')

    @mock.patch('subprocess.check_output')
    def test_hiera__empty(self, mock_sub):
        """Verify None is returned when hiera output is empty string."""
        h = self.create_client('my-config.yml')
        mock_sub.return_value = '""'

        actual_value = h._hiera('some-key')

        self.assertIsNone(actual_value)

    @mock.patch('subprocess.check_output')
    def test_hiera__complex(self, mock_sub):
        """Verify we parse complex data types."""
        h = self.create_client('my-config.yml')
        expected_val = {"key1": "value1",
                        "key2": ["value2a", "value2b"]}
        mock_sub.return_value = ('{ "key1": "value1",'
                                 '  "key2": ["value2a", "value2b"] }')

        actual_value = h._hiera('some-key')

        self.assertEqual(expected_val, actual_value)

    @mock.patch('subprocess.check_output')
    def test_hiera__json_parse_error(self, mock_sub):
        """Verify that we catch json parse errors."""
        h = self.create_client('my-config.yml')
        mock_sub.return_value = '{"notvalid", "json", }'

        with self.assertRaises(hiera.exc.HieraError):
            h._hiera('some-key')

    def test_command(self):
        """Verify command returns expected list."""
        h = self.create_client('my-config.yml')
        expected_command = ['hiera', '--config', 'my-config.yml',
                            '--format', 'json',
                            'some-key']

        actual_command = h._command('some-key')

        self.assertEqual(expected_command, actual_command)

    def test_command__environment(self):
        """Verify command includes environment list."""
        env = {'environment': 'unittest',
               'fqdn':        'ima-superstar',
               }
        h = self.create_client('my-config.yml', **env)
        expected_command = ['hiera', '--config', 'my-config.yml',
                            '--format', 'json', 'some-key',
                            'environment=unittest', 'fqdn=ima-superstar']

        actual_command = h._command('some-key')

        self.assertEqual(expected_command, actual_command)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from hiera import __version__

setup(
    name            ='hiera-py',
    version         =__version__,
    description     ='Python interface for the hiera hierachical database.',
    long_description=open('README.rst', 'r').read(),
    author          ='Thomas Van Doren',
    author_email    ='thomas@thomasvandoren.com',
    maintainer      ='Thomas Van Doren',
    maintainer_email='thomas@thomasvandoren.com',
    url             ='https://hiera-py.readthedocs.org/',
    keywords        =['hiera'],
    license         ='BSD',
    packages        =['hiera'],
    classifiers     =[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        ],
    install_requires = ['simplejson'],
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Python client for Hiera hierachical database."""

from __future__ import print_function, unicode_literals

import json
import logging
import os.path
import subprocess

import hiera.exc

__all__ = ('HieraClient',)


class HieraClient(object):
    __doc__ = __doc__

    def __init__(self, config_filename, hiera_binary='hiera', **kwargs):
        """Create a new instance with the given settings.

        Key value params passed into this will be added to the environment when
        running the hiera client. For example, (environment='developer',
        osfamily='Debian') as keyword args to __init__ would result in hiera
        calls like this:

          hiera --config <config_filename> <key> environment=developer \
            osfamily=Debian

        :param config_filename: Path to the hiera configuration file.
        :param hiera_binary: Path to the hiera binary. Defaults to 'hiera'.
        """
        self.config_filename = config_filename
        self.hiera_binary = hiera_binary
        self.environment = kwargs

        self._validate()
        logging.debug('New Hiera instance: {0}'.format(self))

    def __repr__(self):
        """String representations of Hiera instance."""
        def kv_str(key):
            """Takes an instance attribute and returns a string like:
            'key=value'
            """
            return '='.join((key, repr(getattr(self, key, None))))

        params_list = list(map(kv_str,
                              ['config_filename', 'hiera_binary', 'environment']))
        params_string = ', '.join(params_list)
        return '{0}({1})'.format(self.__class__.__name__, params_string)

    def get(self, key_name):
        """Request the given key from hiera.

        Returns the string version of the key when successful.

        Raises :class:`hiera.exc.HieraError` if the key does not exist or there
        was an error invoking hiera. Raises
        :class:`hiera.exc.HieraNotFoundError` if the hiera CLI binary could not
        be found.

        :param key_name: string key
        :rtype: str value for key or None
        """
        return self._hiera(key_name)

    def _command(self, key_name):
        """Returns a hiera command list that is suitable for passing to
        subprocess calls.

        :param key_name:
        :rtype: list that is hiera command
        """
        cmd = [self.hiera_binary,
               '--config', self.config_filename,
               '--format', 'json',
               key_name]
        cmd.extend(list(map(lambda *env_var: '='.join(*env_var),
                        iter(self.environment.items()))))
        return cmd

    def _hiera(self, key_name):
        """Invokes hiera in a subprocess with the instance environment to query
        for the given key.

        Returns the string version of the key when successful.

        Raises HieraError if the key does not exist or there was an error
        invoking hiera. Raises HieraNotFoundError if the hiera CLI binary could
        not be found.

        :param key_name: string key
        :rtype: str value for key or None
        """
        hiera_command = self._command(key_name)
        output = None
        try:
            output = subprocess.check_output(
                hiera_command, stderr=subprocess.STDOUT)
        except OSError as ex:
            raise hiera.exc.HieraNotFoundError(
                'Could not find hiera binary at: {0}'.format(
                    self.hiera_binary))
        except subprocess.CalledProcessError as ex:
            raise hiera.exc.HieraError(
                'Failed to retrieve key {0}. exit code: {1} '
                'message: {2} console output: {3}'.format(
                    key_name, ex.returncode, ex, ex.output))
        else:
            try:
                value = json.loads(output.strip())
            except ValueError as ex:
                raise hiera.exc.HieraError(
                    'Failed to convert hiera output {0} for key {1}. '
                    'message: {2}'.format(
                        output.strip(), key_name, ex))
            if not value:
                return None
            else:
                return value

    def _validate(self):
        """Validate the instance attributes. Raises HieraError if issues are
        found.
        """
        if not os.path.isfile(self.config_filename):
            raise hiera.exc.HieraError(
                'Hiera configuration file does not exist '
                'at: {0}'.format(self.config_filename))
